<?php

/**
 * @see Zend_Service_Abstract
 */
include_once 'Zend/Service/Abstract.php';

/**
 * @category Zext
 * @package Service
 * @subpackage Translink
 * @author Dustin Thomson <dustin.thomson@gmail.com>
 */
class Zext_Service_Translink extends Zend_Service_Abstract 
{
	/**
	 * An agent string to mimic the iphone.
	 * This is required because the site was designed for iphone use
	 * 
	 * @var string
	 */
	protected $_iphoneAgent = "Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1A543 Safari/419.3";
	
	/**
	 * Base url for the api
	 * @var string
	 */
	protected $_uri = 'http://m.translink.ca/api/';
	
	/**
	 * Url of the route search endpoint
	 * @var string
	 */
	protected $_routeSearchUri = 'http://m.translink.ca/api/routes/';
	
	/**
	 * Url of the route direction search endpoint
	 * @var string
	 */
	protected $_routeDirectionUri = 'http://m.translink.ca/api/directions/';
	
	/**
	 * url of the route stop endpoint
	 * @var string
	 */
	protected $_routeStopsUri = 'http://m.translink.ca/api/stops/';
	
	/**
	 * URL of the stop details endpoint
	 * @var string
	 */
	protected $_stopDetailsUri = 'http://m.translink.ca/api/stops/';
	
	/**
	 * URL of the stop location endpoint
	 * @var string
	 */
	protected $_stopLocationUri = 'http://m.translink.ca/api/kml/stop/';
	
	/**
	 * A list of direction so we can validate the {@link getRouteStops()} function 
	 *
	 * @var array
	 */
	protected $_directions = array(
		'north',
		'south',
		'east',
		'west'
	);
	
	/**
	 * The translink mobile site is ment to be viewed using an Iphone, therefore it may be less conspicuious if we use that useragent
	 *
	 * @param boolean $fakeIphoneAgent
	 */
	public function __construct($fakeIphoneAgent = true)
	{
		if($fakeIphoneAgent){
			self::getHttpClient()->setConfig(array(
				'useragent' => 'Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1A543 Safari/419.3'
			));
		}
	}
	
	/**
	 * Gets the routes that match the route number parameter.
	 * This function may return more than one result if there is more than one potential match
	 * @example $number = 10 will return all routes with 10 in their route number.
	 *
	 * @param int $routeNumber	[OPTIONAL] If this is specified, it acts as a search for the route number
	 * @param array
	 */
	public function getRoutes($route = null)
	{
		$client = self::getHttpClient()->resetParameters();
		$client->setUri($this->_routeSearchUri);
		
		$client->setParameterGet('f', 'json');
		
		if($route != null){
			$client->setParameterGet(array(
				'q' => $route
			));
		}
		
		$result = $this->_processGetRouteResponse($client->request());
		
		return $result;
	}
	
	/**
	 * Processes the result that comes back from the getRoutes() function
	 *
	 * @param Zend_Http_Response $response
	 * @return array
	 */
	protected function _processGetRouteResponse(Zend_Http_Response $response)
	{
		require_once 'Zend/Json.php';
		
		$json = $response->getBody();
		$resultArray = Zend_Json::decode($json);
		
		if(!is_array($resultArray)){
			include_once 'Zext/Service/Translink/Exception.php';
			throw new Zext_Service_Translink_Exception('Failed to parse response body into an array');
		}
		
		$returnArray = array();
		foreach ($resultArray as $route) {
			//Will make $resultArray[<routeNumber>] = <routeName>
			$returnArray[$route[0]] = trim($route[1]);
		}
		
		return $returnArray;
	}
	
	/**
	 * Gets the name and direction of the route.
	 * Returns an array whose key is the direction and whose value is the name of the direction
	 *
	 * @param string $route	The Route number
	 * @return array
	 */
	public function getRouteDirections($route)
	{
		$client = self::getHttpClient()->resetParameters();
		$client->setUri($this->_routeDirectionUri);
		
		$client->setParameterGet(array(
			'f' => 'json',
			'r' => $this->_formatRouteNumber($route)
		));
		
		return $this->_processGetRouteDirectionsResponse($client->request());
	}
	
	/**
	 * processes the result of {@link getRouteDirections} and returns a formatted array
	 *
	 * @param Zend_Http_Response $response
	 * @return array
	 */
	protected function _processGetRouteDirectionsResponse(Zend_Http_Response $response)
	{
		require_once 'Zend/Json.php';
		
		$json = $response->getBody();
		$resultArray = Zend_Json::decode($json);
		
		if(!is_array($resultArray)){
			include_once 'Zext/Service/Translink/Exception.php';
			throw new Zext_Service_Translink_Exception('Failed to parse response body into an array');
		}
		
		$returnArray = array();
		foreach ($resultArray as $route) {
			//Will make $resultArray[<direction>] = <destination name>
			$returnArray[$route[0]] = trim($route[1]);
		}
		
		return $returnArray;
	}
	
	/**
	 * Gets all the stops along a given route in a given direction.
	 * 
	 * @param string $route
	 * @param string $direction
	 * @return array
	 */
	public function getRouteStops($route, $direction)
	{
		$client = self::getHttpClient()->resetParameters();
		$client->setUri($this->_routeStopsUri);
		
		$direction = strtolower($direction);
		
		if(!in_array($direction, $this->_directions)){
			include_once 'Zext/Service/Translink/Exception.php';
			throw new Zext_Service_Translink_Exception('Direction Must be one of ' . join(',', $this->_directions) . '. "' . $direction . '" specified');
		}
		
		$client->setParameterGet(array(
			'f' => 'json',
			'r' => $this->_formatRouteNumber($route),
			'd' => $direction
		));
		
		return $this->_processGetRouteStopsResponse($client->request());
	}
	
	/**
	 * processes the result of {@link getRouteDirections} and returns a formatted array
	 *
	 * @param Zend_Http_Response $response
	 * @return array
	 */
	protected function _processGetRouteStopsResponse(Zend_Http_Response $response)
	{
		require_once 'Zend/Json.php';
		
		$json = $response->getBody();
		$resultArray = Zend_Json::decode($json);
		
		if(!is_array($resultArray)){
			include_once 'Zext/Service/Translink/Exception.php';
			throw new Zext_Service_Translink_Exception('Failed to parse response body into an array');
		}
		
		$returnArray = array();
		foreach ($resultArray as $stop) {
			$returnArray[$stop[0]] = array(
				'number' => $stop[0],
				'name' => $stop[1],
				'times' => array_slice($stop, 2)
			);
		}
		
		return $returnArray;
	}
	
	/**
	 * Gets all the routes that stop at the specified stop, complete with stop times.
	 *
	 * @param int $stopNumber
	 * @return array
	 */
	public function getStopDetails($stopNumber)
	{
		$client = self::getHttpClient()->resetParameters();
		$client->setUri($this->_routeStopsUri);
		
		$client->setParameterGet(array(
			'f' => 'json',
			's' => $stopNumber,
		));
		
		return $this->_processGetStopDetailsResponse($client->request());
	}
	
	/**
	 * processes the result of {@link getStopDetails} and returns a formatted array
	 *
	 * @param Zend_Http_Response $response
	 * @return array
	 */
	protected function _processGetStopDetailsResponse(Zend_Http_Response $response)
	{
		require_once 'Zend/Json.php';
		
		$json = $response->getBody();
		$resultArray = Zend_Json::decode($json);
		
		if(!is_array($resultArray)){
			include_once 'Zext/Service/Translink/Exception.php';
			throw new Zext_Service_Translink_Exception('Failed to parse response body into an array');
		}
		
		$returnArray = array();
		foreach ($resultArray as $route) {
			$routeArray = explode(' ', $route[0]);
			
			switch ($routeArray[1]){
				case '(W)':
					$direction = 'west';
				break;
				case '(E)':
					$direction = 'east';
				break;
				case '(S)':
					$direction = 'south';
				break;
				case '(N)':
					$direction = 'north';
				break;
			}
			
			$returnArray[$routeArray[0]] = array(
				'route' => $routeArray[0],
				'direction' => $direction,
				'name' => $route[1],
				'times' => array_slice($route, 2)
			);
		}
		
		return $returnArray;
	}
	
	/**
	 * Gets the GPS coordinates of a given stop number.
	 * 
	 * @param mixed $stopNumber
	 * @return array
	 */
	public function getStopLocation($stopNumber)
	{
		$client = self::getHttpClient();
		$client->setUri($this->_stopLocationUri . $stopNumber);
		return $this->_processGetStopLocationResponse($client->request());
	}
	
	/**
	 * Parses the response of {@link getStopLocation} and returns a formatted array
	 * 
	 * @param Zend_Http_Response $response
	 * @return array
	 */
	protected function _processGetStopLocationResponse(Zend_Http_Response $response)
	{
		require_once 'Zend/Dom/Query.php';
		
		$xml = $response->getBody();
		
		$query = new Zend_Dom_Query();
		$query->setDocumentXml($xml);
		
		$result = $query->queryXpath('.');
		
		//Set query namespace
		$ns = $result->current()->getAttribute('xmlns');
		
		$doc = new DOMDocument();
		$doc->loadXML($query->getDocument());
		$xpath = new DOMXPath($doc);
		$xpath->registerNamespace('d', $ns);
		
		$result = $xpath->query('//d:name');
		$stopName = $result->item(0)->nodeValue;
		
		$result = $xpath->query('//d:coordinates');
		$coordinates = $result->item(0)->nodeValue;
		$coordainteArray = explode(',', $coordinates);
		
		return array(
			'name' => $stopName,
			'lat' => floatval($coordainteArray[1]),
			'lon' => floatval($coordainteArray[0])
		);
	}
	
	/**
	 * Gets the 10 closest stops to the passed location
	 *
	 * @param float $lat
	 * @param float $lng
	 * @return array
	 */
	public function getLocationStops($lat, $lng)
	{
		$lat = floatval($lat);
		$lng = floatval($lng);
		
		$client = new Zend_Http_Client();
		$client->setUri($this->_routeStopsUri);
		
		$client->setParameterGet(array(
			'f' => 'json',
			'lng' => sprintf("%.5F", $lng),
			'lat' => sprintf("%.5F", $lat)
		));
		
		return $this->_processGetRouteStopsResponse($client->request());
	}
	
	/**
	 * Formats the route number.
	 * Ensures that if the route is a number, will be a 3 digit number, otherwise just trims the string
	 *
	 * @param mixed $route
	 * @return string
	 */
	protected function _formatRouteNumber($route)
	{
		if(is_numeric($route)){
			return sprintf("%03d", $route);
		}else{
			return trim($route);
		}
	}
}